@@ -2,7 +2,7 @@
 /*
  * gsf-libxml.c :
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2005 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -32,9 +32,163 @@
 
 static GObjectClass *parent_class;
 
+static gint
+glade_enum_from_string (GType type, const char *string)
+{
+    GEnumClass *eclass;
+    GEnumValue *ev;
+    gchar *endptr;
+    gint ret = 0;
+
+    ret = strtoul(string, &endptr, 0);
+    if (endptr != string) /* parsed a number */
+	return ret;
+
+    eclass = g_type_class_ref(type);
+    ev = g_enum_get_value_by_name(eclass, string);
+    if (!ev) ev = g_enum_get_value_by_nick(eclass, string);
+    if (ev)  ret = ev->value;
+
+    g_type_class_unref(eclass);
+
+    return ret;
+}
+
+static guint
+glade_flags_from_string (GType type, const char *string)
+{
+    GFlagsClass *fclass;
+    gchar *endptr, *prevptr;
+    guint i, j, ret = 0;
+    char *flagstr;
+
+    ret = strtoul(string, &endptr, 0);
+    if (endptr != string) /* parsed a number */
+	return ret;
+
+    fclass = g_type_class_ref(type);
+
+
+    flagstr = g_strdup (string);
+    for (ret = i = j = 0; ; i++) {
+	gboolean eos;
+
+	eos = flagstr [i] == '\0';
+	
+	if (eos || flagstr [i] == '|') {
+	    GFlagsValue *fv;
+	    const char  *flag;
+	    gunichar ch;
+
+	    flag = &flagstr [j];
+            endptr = &flagstr [i];
+
+	    if (!eos) {
+		flagstr [i++] = '\0';
+		j = i;
+	    }
+
+            /* trim spaces */
+	    for (;;)
+	      {
+		ch = g_utf8_get_char (flag);
+		if (!g_unichar_isspace (ch))
+		  break;
+		flag = g_utf8_next_char (flag);
+	      }
+
+	    while (endptr > flag)
+	      {
+		prevptr = g_utf8_prev_char (endptr);
+		ch = g_utf8_get_char (prevptr);
+		if (!g_unichar_isspace (ch))
+		  break;
+		endptr = prevptr;
+	      }
+
+	    if (endptr > flag)
+	      {
+		*endptr = '\0';
+		fv = g_flags_get_value_by_name (fclass, flag);
+
+		if (!fv)
+		  fv = g_flags_get_value_by_nick (fclass, flag);
+
+		if (fv)
+		  ret |= fv->value;
+		else
+		  g_warning ("Unknown flag: '%s'", flag);
+	      }
+
+	    if (eos)
+		break;
+	}
+    }
+    
+    g_free (flagstr);
+
+    g_type_class_unref(fclass);
+
+    return ret;
+}
+
+gboolean
+gsf_xml_gvalue_from_str (GValue *res, GType t, char const *str)
+{
+	g_return_val_if_fail (res != NULL, FALSE);
+	g_return_val_if_fail (str != NULL, FALSE);
+
+	g_value_init (res, t);
+	switch (t) {
+	case G_TYPE_CHAR:
+		g_value_set_char (res, str[0]);
+		break;
+	case G_TYPE_UCHAR:
+		g_value_set_uchar (res, (guchar)str[0]);
+		break;
+	case G_TYPE_BOOLEAN:
+		g_value_set_boolean (res, 
+			g_ascii_tolower (str[0]) == 't' ||
+			g_ascii_tolower (str[0]) == 'y' ||
+			strtol (str, NULL, 0));
+		break;
+	case G_TYPE_INT:
+		g_value_set_int (res, strtol (str, NULL, 0));
+		break;
+	case G_TYPE_UINT:
+		g_value_set_uint (res, strtoul (str, NULL, 0));
+		break;
+	case G_TYPE_LONG:
+		g_value_set_long (res, strtol (str, NULL, 0));
+		break;
+	case G_TYPE_ULONG:
+		g_value_set_ulong (res, strtoul (str, NULL, 0));
+		break; 
+	case G_TYPE_ENUM:
+		g_value_set_enum (res, glade_enum_from_string (t, str));
+		break;
+	case G_TYPE_FLAGS:
+		g_value_set_flags (res, glade_flags_from_string (t, str));
+		break;
+	case G_TYPE_FLOAT:
+		g_value_set_float (res, g_strtod (str, NULL));
+		break;
+	case G_TYPE_DOUBLE:
+		g_value_set_double (res, g_strtod (str, NULL));
+		break;
+	case G_TYPE_STRING:
+		g_value_set_string (res, str);
+		break;
+
+	default:
+		return FALSE;
+	}
+	return TRUE;
+}
+
 /* Note: libxml erroneously declares the length argument as int.  */
 static int
-gsf_libxml_read (void *context, char *buffer, int len)
+gsf_libxml_read (void *context, guint8 *buffer, int len)
 {
 	gsf_off_t remaining = gsf_input_remaining ((GsfInput *)context);
 	guint8* res;
@@ -164,23 +318,45 @@ gsf_xmlDocFormatDump (GsfOutput *output, xmlDocPtr cur, char const *encoding,
 
 /***************************************************************************/
 
+typedef struct {
+	GsfXMLInNode pub;
+	/* internal state */
+	GSList *groups;
+} GsfXMLInNodeInternal;
 struct _GsfXMLInDoc {
-	GsfXMLInNode	*root;
-	GsfXMLInNS	*ns;
+	GsfXMLInNodeInternal const *root_node;
+	GHashTable      *symbols; /* GsfXMLInNodeInternal hashed by id */
+	GsfXMLInNS const*ns;
 	GPtrArray	*ns_by_id;
 	GsfXMLInUnknownFunc	unknown_handler;
 };
+typedef struct {
+	GsfXMLIn	pub;
 
+	GsfXMLInNS   const *default_ns;	/* optionally NULL */
+	GSList	 	   *ns_stack;
+	GHashTable	   *ns_prefixes;
+	GPtrArray	   *ns_by_id;
+	gint	  	    unknown_depth; /* handle recursive unknown tags */
+	gboolean	    from_unknown_handler;
+
+	GSList	 	   *extension_stack;
+} GsfXMLInInternal;
 typedef struct {
 	char    *tag;
 	unsigned taglen;
 	unsigned ref_count;
 } GsfXMLInNSInstance;
-
 typedef struct {
 	GsfXMLInNS const *ns;
 	GSList *elem;
 } GsfXMLInNodeGroup;
+typedef struct {
+	GsfXMLInDoc const *old_doc;
+	gpointer	   old_state;
+	GsfXMLInExtDtor    dtor;
+	gboolean	   from_unknown;
+} GsfXMLInExtension;
 
 static char const *
 node_name (GsfXMLInNode const *node)
@@ -189,17 +365,72 @@ node_name (GsfXMLInNode const *node)
 }
 
 static void
-gsf_xml_in_start_element (GsfXMLIn *state, xmlChar const *name, xmlChar const **attrs)
+push_child (GsfXMLInInternal *state, GsfXMLInNode const *node, GsfXMLInNS const *default_ns,
+	    xmlChar const **attrs, GsfXMLInExtension *ext)
+{
+	if (node->has_content == GSF_XML_CONTENT &&
+	    state->pub.content->len > 0) {
+		g_warning ("too lazy to support nested unshared content for now.  We'll add it for 2.0");
+	}
+	state->pub.node_stack	= g_slist_prepend (state->pub.node_stack,
+		(gpointer)state->pub.node);
+	state->ns_stack		= g_slist_prepend (state->ns_stack,
+		(gpointer)state->default_ns);
+	state->extension_stack	= g_slist_prepend (state->extension_stack, ext);
+	state->pub.node = node;
+	state->default_ns = default_ns;
+	if (node->start != NULL)
+		node->start (&state->pub, attrs);
+}
+
+static gboolean
+lookup_child (GsfXMLInInternal *state, GsfXMLInNS const *default_ns,
+	      GsfXMLInNSInstance *inst,
+	      GSList *groups, xmlChar const *name, xmlChar const **attrs)
 {
-	GSList *ptr, *elem;
-	GsfXMLInNSInstance *inst;
 	GsfXMLInNodeGroup  *group;
 	GsfXMLInNode	   *node;
+	GSList *elem, *ptr;
+	char const *tmp;
+
+	for (ptr = groups ; ptr != NULL ; ptr = ptr->next) {
+		group = ptr->data;
+		/* does the namespace match */
+		if (group->ns != NULL && group->ns != default_ns) {
+			g_return_val_if_fail (state->ns_by_id->len > group->ns->ns_id, FALSE);
+			inst = g_ptr_array_index (state->ns_by_id, group->ns->ns_id);
+			if (inst == NULL || 0 != strncmp (name, inst->tag, inst->taglen))
+				continue;
+			tmp = name + inst->taglen;
+		} else {
+#if 0
+			g_return_if_fail (state->ns_by_id->len > group->ns->ns_id);
+			inst = g_ptr_array_index (state->ns_by_id, group->ns->ns_id);
+			g_warning ("accepted ns = '%s' looking for '%s'", inst->tag, name);
+#endif
+			tmp = name;
+		}
+		for (elem = group->elem ; elem != NULL ; elem = elem->next) {
+			node = elem->data;
+			if (node->name == NULL || !strcmp (tmp, node->name)) {
+				push_child (state, node, default_ns, attrs, NULL);
+				return TRUE;
+			}
+		}
+	}
+	return FALSE;
+}
+
+static void
+gsf_xml_in_start_element (GsfXMLInInternal *state, xmlChar const *name, xmlChar const **attrs)
+{
+	GsfXMLInNSInstance *inst;
 	GsfXMLInNS const   *ns, *default_ns = state->default_ns;
+	GsfXMLInNodeInternal const *node;
 	xmlChar const **ns_ptr;
+	GSList *ptr;
 	char const *tmp;
 	int i;
-	gboolean check_unknown_handler = TRUE;
 
 	/* Scan for namespace declarations.  Yes it is ugly to have the api
 	 * flag that its children can declare namespaces. However, given that a
@@ -208,8 +439,8 @@ gsf_xml_in_start_element (GsfXMLIn *state, xmlChar const *name, xmlChar const **
 	 * eg <gnm:Workbook xmlns:gnm="www.gnumeric.org"/> we can not know
 	 * that we are in node 'Workbook' without recognizing ns=gnm, which we
 	 * would not do unless we checked for a namespace */
-	ns = state->doc->ns;
-	if (ns != NULL && state->node->check_children_for_ns) {
+	ns = state->pub.doc->ns;
+	if (ns != NULL && state->pub.node->check_children_for_ns) {
 		for (ns_ptr = attrs; ns_ptr != NULL && ns_ptr[0] && ns_ptr[1] ; ns_ptr += 2) {
 			if (strncmp (*ns_ptr, "xmlns", 5))
 				continue;
@@ -246,140 +477,131 @@ gsf_xml_in_start_element (GsfXMLIn *state, xmlChar const *name, xmlChar const **
 		}
 	}
 
-lookup_child :
-	for (ptr = state->node->groups ; ptr != NULL ; ptr = ptr->next) {
-		group = ptr->data;
-		/* does the namespace match */
-		if (group->ns != NULL && group->ns != default_ns) {
-			g_return_if_fail (state->ns_by_id->len > group->ns->ns_id);
-			inst = g_ptr_array_index (state->ns_by_id, group->ns->ns_id);
-			if (inst == NULL || 0 != strncmp (name, inst->tag, inst->taglen))
-				continue;
-			tmp = name + inst->taglen;
-		} else {
-#if 0
-			g_return_if_fail (state->ns_by_id->len > group->ns->ns_id);
-			inst = g_ptr_array_index (state->ns_by_id, group->ns->ns_id);
-			g_warning ("accepted ns = '%s' looking for '%s'", inst->tag, name);
-#endif
-			tmp = name;
-		}
-		for (elem = group->elem ; elem != NULL ; elem = elem->next) {
-			node = elem->data;
-			if (node->name == NULL || !strcmp (tmp, node->name)) {
-				if (node->has_content == GSF_XML_CONTENT &&
-				    state->content->len > 0) {
-					g_warning ("too lazy to support nested unshared content for now.  We'll add it for 2.0");
-				}
-				state->state_stack = g_slist_prepend (state->state_stack,
-								      (gpointer)state->node);
-				state->ns_stack = g_slist_prepend (state->ns_stack,
-								   (gpointer)state->default_ns);
-				state->node = node;
-				state->default_ns = default_ns;
-				if (node->start != NULL)
-					node->start (state, attrs);
-				return;
-			}
-		}
+	node = (GsfXMLInNodeInternal const *) state->pub.node;
+	if (lookup_child (state, default_ns, inst, node->groups, name, attrs))
+		return;
+
+	/* useful for <Data><b><i><u></u></i></b></Data> where all of the markup can nest */
+	ptr = state->pub.node_stack;
+	for (; ptr != NULL && node->pub.share_children_with_parent; ptr = ptr->next) {
+		node = ptr->data;
+		if (lookup_child (state, default_ns, inst, node->groups, name, attrs))
+			return;
 	}
 
-	if (check_unknown_handler) {
-		check_unknown_handler = FALSE; /* only loop once */
-		if (state->doc->unknown_handler != NULL &&
-		    (state->doc->unknown_handler) (state, name, attrs))
-			goto lookup_child;
+	if (state->pub.doc->unknown_handler != NULL) {
+		gboolean handled;
+		state->from_unknown_handler = TRUE;
+		handled = (state->pub.doc->unknown_handler) (&state->pub, name, attrs);
+		state->from_unknown_handler = FALSE;
+		if (handled)
+			return;
 	}
-	if (state->unknown_depth++)
+
+	if (state->unknown_depth++ > 0)
 		return;
-	g_warning ("Unexpected element '%s' in state %s.", name, node_name (state->node));
-	{
-		GSList *ptr;
-		GsfXMLInNode *node;
-		ptr = state->state_stack = g_slist_reverse (state->state_stack);
-		for (;ptr != NULL && ptr->next != NULL; ptr = ptr->next) {
-			node = ptr->data;
-			if (node != NULL) {
-#ifdef __GNUC__
-#warning if we really want this do we also want namespaces ?
-#endif
-				g_print ("%s", node_name (node));
-				if (ptr->next != NULL && ptr->next->data != NULL)
-					g_print (" -> ");
-			}
+	g_warning ("Unexpected element '%s' in state %s.", name, node_name (state->pub.node));
+
+	ptr = state->pub.node_stack = g_slist_reverse (state->pub.node_stack);
+	for (;ptr != NULL && ptr->next != NULL; ptr = ptr->next) {
+		node = ptr->data;
+		if (node != NULL) {
+/* FIXME FIXME FIXME if we really want this do we also want namespaces ? */
+			g_print ("%s", node_name (&node->pub));
+			if (ptr->next != NULL && ptr->next->data != NULL)
+				g_print (" -> ");
 		}
-		state->state_stack = g_slist_reverse (state->state_stack);
 	}
+	state->pub.node_stack = g_slist_reverse (state->pub.node_stack);
 }
 
 static void
-gsf_xml_in_end_element (GsfXMLIn *state,
+gsf_xml_in_end_element (GsfXMLInInternal *state,
 			G_GNUC_UNUSED xmlChar const *name)
 {
+	GsfXMLInExtension *ext;
+	gboolean was_overlay;
+
 	if (state->unknown_depth > 0) {
 		state->unknown_depth--;
 		return;
 	}
 
-	g_return_if_fail (state->state_stack != NULL);
-	g_return_if_fail (state->ns_stack != NULL);
+	do {
+		g_return_if_fail (state->pub.node_stack != NULL);
+		g_return_if_fail (state->ns_stack != NULL);
+		g_return_if_fail (state->extension_stack != NULL);
 
-	if (state->node->end)
-		state->node->end (state, NULL);
-	if (state->node->has_content == GSF_XML_CONTENT)
-		g_string_truncate (state->content, 0);
+		if (state->pub.node->end)
+			state->pub.node->end (&state->pub, NULL);
+		if (state->pub.node->has_content == GSF_XML_CONTENT)
+			g_string_truncate (state->pub.content, 0);
 
-	/* pop the state stack */
-	state->node	   = state->state_stack->data;
-	state->state_stack = g_slist_remove (state->state_stack, state->node);
-	state->default_ns = state->ns_stack->data;
-	state->ns_stack   = g_slist_remove (state->ns_stack, state->default_ns);
+		/* pop the state stack */
+		ext = state->extension_stack->data;
+		state->extension_stack	= g_slist_remove (state->extension_stack, ext);
+		state->pub.node		= state->pub.node_stack->data;
+		state->pub.node_stack	= g_slist_remove (state->pub.node_stack, state->pub.node);
+		state->default_ns	= state->ns_stack->data;
+		state->ns_stack		= g_slist_remove (state->ns_stack, state->default_ns);
+
+		if (NULL != ext) {
+			was_overlay = !ext->from_unknown;
+			if (ext->dtor)
+				(ext->dtor) (&state->pub, ext->old_state);
+			state->pub.user_state = ext->old_state;
+			state->pub.doc = ext->old_doc;
+			g_free (ext);
+		} else
+			was_overlay = FALSE;
+	} while (was_overlay);
 }
 
 static void
-gsf_xml_in_characters (GsfXMLIn *state, const xmlChar *chars, int len)
+gsf_xml_in_characters (GsfXMLInInternal *state, xmlChar const *chars, int len)
 {
-	if (state->node->has_content != GSF_XML_NO_CONTENT)
-		g_string_append_len (state->content, chars, len);
+	if (state->pub.node->has_content != GSF_XML_NO_CONTENT)
+		g_string_append_len (state->pub.content, chars, len);
 }
 
 static xmlEntityPtr
-gsf_xml_in_get_entity (G_GNUC_UNUSED GsfXMLIn *state, const xmlChar *name)
+gsf_xml_in_get_entity (G_GNUC_UNUSED GsfXMLInInternal *state, xmlChar const *name)
 {
 	return xmlGetPredefinedEntity (name);
 }
 
 static void
-gsf_xml_in_start_document (GsfXMLIn *state)
+gsf_xml_in_start_document (GsfXMLInInternal *state)
 {
-	state->node = state->doc->root;
-	state->unknown_depth = 0;
-	state->state_stack = NULL;
-	state->ns_stack    = NULL;
-	state->default_ns  = NULL;
-	state->ns_by_id    = g_ptr_array_new ();
-	state->ns_prefixes = g_hash_table_new_full (g_str_hash, g_str_equal,
-		NULL, g_free);
+	state->pub.node = &state->pub.doc->root_node->pub;
+	state->unknown_depth	= 0;
+	state->pub.node_stack	= NULL;
+	state->extension_stack	= NULL;
+	state->ns_stack		= NULL;
+	state->default_ns	= NULL;
+	state->ns_by_id		= g_ptr_array_new ();
+	state->ns_prefixes	= g_hash_table_new_full (
+		g_str_hash, g_str_equal, NULL, g_free);
 }
 
 static void
-gsf_xml_in_end_document (GsfXMLIn *state)
+gsf_xml_in_end_document (GsfXMLInInternal *state)
 {
-	g_string_free (state->content, TRUE);
-	state->content = NULL;
-
-	g_return_if_fail (state->node == state->doc->root);
-	g_return_if_fail (state->unknown_depth == 0);
+	g_string_free (state->pub.content, TRUE);
+	state->pub.content = NULL;
 
 	g_ptr_array_free (state->ns_by_id, TRUE);
 	state->ns_by_id = NULL;
 
 	g_hash_table_destroy (state->ns_prefixes);
 	state->ns_prefixes = NULL;
+
+	g_return_if_fail (state->pub.node == &state->pub.doc->root_node->pub);
+	g_return_if_fail (state->unknown_depth == 0);
 }
 
 static void
-gsf_xml_in_warning (G_GNUC_UNUSED GsfXMLIn *state, char const *msg, ...)
+gsf_xml_in_warning (G_GNUC_UNUSED GsfXMLInInternal *state, char const *msg, ...)
 {
 	va_list args;
 
@@ -389,7 +611,7 @@ gsf_xml_in_warning (G_GNUC_UNUSED GsfXMLIn *state, char const *msg, ...)
 }
 
 static void
-gsf_xml_in_error (G_GNUC_UNUSED GsfXMLIn *state, char const *msg, ...)
+gsf_xml_in_error (G_GNUC_UNUSED GsfXMLInInternal *state, char const *msg, ...)
 {
 	va_list args;
 
@@ -399,7 +621,7 @@ gsf_xml_in_error (G_GNUC_UNUSED GsfXMLIn *state, char const *msg, ...)
 }
 
 static void
-gsf_xml_in_fatal_error (G_GNUC_UNUSED GsfXMLIn *state, char const *msg, ...)
+gsf_xml_in_fatal_error (G_GNUC_UNUSED GsfXMLInInternal *state, char const *msg, ...)
 {
 	va_list args;
 
@@ -447,85 +669,117 @@ static xmlSAXHandler gsfXMLInParser = {
 #endif
 };
 
+static void
+gsf_xml_in_node_internal_free (GsfXMLInNodeInternal *node)
+{
+	GSList *ptr;
+	GsfXMLInNodeGroup *group;
+
+	for (ptr = node->groups; ptr != NULL ; ptr = ptr->next) {
+		group = ptr->data;
+		g_slist_free (group->elem);
+		g_free (group);
+	}
+	g_slist_free (node->groups);
+	node->groups = NULL;
+	g_free (node);
+}
+
+/**
+ * gsf_xml_in_doc_free :
+ * @doc :
+ *
+ * Free up resources
+ **/
+void
+gsf_xml_in_doc_free (GsfXMLInDoc *doc)
+{
+	g_return_if_fail (doc != NULL);
+	g_return_if_fail (doc->symbols != NULL);
+	g_return_if_fail (doc->ns_by_id != NULL);
+
+	g_hash_table_destroy (doc->symbols);
+	g_ptr_array_free (doc->ns_by_id, TRUE);
+	/* poison the well just in case */
+	doc->symbols   = NULL;
+	doc->ns_by_id  = NULL;
+	doc->root_node = NULL;
+	g_free (doc);
+}
+
 /**
  * gsf_xml_in_doc_new :
- * @root : an array of node descriptors
+ * @nodes : an array of node descriptors
  * @ns : an array of namespace identifiers
  *
- * Put the nodes in the NULL terminated array starting at @root and the name
- * spaces in the NULL terminated array starting at @ns together.  Link them up
- * and prepare the static data structures necessary to validate a doument based
- * on that description.
+ * Combine the nodes in the NULL terminated array starting at @nodes with the
+ * name spaces in the NULL terminated array starting at @ns.  Prepare the
+ * data structures necessary to validate a doument based on that description.
  *
  * Returns NULL on error
  **/
 GsfXMLInDoc *
-gsf_xml_in_doc_new (GsfXMLInNode *root, GsfXMLInNS *ns)
+gsf_xml_in_doc_new (GsfXMLInNode const *nodes, GsfXMLInNS const *ns)
 {
 	GsfXMLInDoc  *doc;
+	GsfXMLInNode const *e_node;
+	GsfXMLInNodeInternal *tmp, *node;
 	unsigned i;
 
-	if (root->parent_initialized)
-		return NULL;
+	g_return_val_if_fail (nodes != NULL, NULL);
 
 	doc = g_new0 (GsfXMLInDoc, 1);
-	doc->root     = root;
-	doc->ns       = ns;
-	doc->ns_by_id = g_ptr_array_new ();
+	doc->root_node = NULL;
+	doc->symbols   = g_hash_table_new_full (g_str_hash, g_str_equal,
+		NULL, (GDestroyNotify) gsf_xml_in_node_internal_free);
+	doc->ns        = ns;
+	doc->ns_by_id  = g_ptr_array_new ();
 
 	/* Add namespaces to an idex */
-	if (ns != NULL) {
+	if (ns != NULL)
 		for (i = 0; ns[i].uri != NULL ; i++) {
 			if (ns[i].ns_id >= doc->ns_by_id->len)
 				g_ptr_array_set_size  (doc->ns_by_id, ns[i].ns_id+1);
-			g_ptr_array_index (doc->ns_by_id, ns[i].ns_id) = ns+i;
+			g_ptr_array_index (doc->ns_by_id, ns[i].ns_id) = (gpointer)(ns+i);
 		}
-	}
-	gsf_xml_in_doc_extend (doc, root);
-	return doc;
-}
 
-void
-gsf_xml_in_doc_extend (GsfXMLInDoc  *doc,
-		       GsfXMLInNode *extension_nodes)
-{
-	GsfXMLInNode *tmp, *real_node, *node;
-	GHashTable   *symbols = g_hash_table_new (g_str_hash, g_str_equal);
-
-	g_return_if_fail (doc != NULL);
-	g_return_if_fail (extension_nodes != NULL);
-
-	if (extension_nodes->parent_initialized)
-		return;
-	for (node = extension_nodes; node->id != NULL ; node++ ) {
-		g_return_if_fail (!node->parent_initialized);
-
-		tmp = g_hash_table_lookup (symbols, node->id);
-		if (tmp != NULL) {
+	for (e_node = nodes; e_node->id != NULL ; e_node++ ) {
+		node = g_hash_table_lookup (doc->symbols, e_node->id);
+		if (node != NULL) {
 			/* if its empty then this is just a recusion */
-			if (node->start != NULL || node->end != NULL ||
-			    node->has_content != GSF_XML_NO_CONTENT || node->user_data.v_int != 0) {
+			if (e_node->start != NULL || e_node->end != NULL ||
+			    e_node->has_content != GSF_XML_NO_CONTENT ||
+			    e_node->user_data.v_int != 0) {
 				g_warning ("ID '%s' has already been registered.\n"
-					   "The additional decls should not specify start,end,content,data", node->id);
-				return;
+					   "The additional decls should not specify start,end,content,data", e_node->id);
+				continue;
 			}
-			real_node = tmp;
 		} else {
-			/* be anal, the macro probably initialized this, but do it just in case */
+			node = g_new0 (GsfXMLInNodeInternal, 1);
+			node->pub = *e_node;
+			/* WARNING VILE HACK :
+			 * The api in 1.8.2 passed has_content as a boolean.
+			 * Too many things still use that to change yet.  We
+			 * edit the bool here to be GSF_CONTENT_NONE or
+			 * GSF_XML_CONTENT and try to ignore SHARED_CONTENT */
+			if (node->pub.has_content != 0 &&
+			    node->pub.has_content != GSF_XML_SHARED_CONTENT)
+				node->pub.has_content = GSF_XML_CONTENT;
 			node->groups = NULL;
-			g_hash_table_insert (symbols, (gpointer)node->id, node);
-			real_node = node;
+			g_hash_table_insert (doc->symbols,
+				(gpointer)node->pub.id, node);
 		}
+		if (e_node == nodes) /* first valid node is the root */
+			doc->root_node = node;
 
-		tmp = g_hash_table_lookup (symbols, node->parent_id);
+		tmp = g_hash_table_lookup (doc->symbols, node->pub.parent_id);
 		if (tmp != NULL) {
 			GSList *ptr;
 			GsfXMLInNodeGroup *group = NULL;
 			GsfXMLInNS const *ns = NULL;
 			
-			ns = (real_node->ns_id < 0) ? NULL
-				: g_ptr_array_index (doc->ns_by_id,
-						     real_node->ns_id);
+			ns = (node->pub.ns_id < 0) ? NULL
+				: g_ptr_array_index (doc->ns_by_id, node->pub.ns_id);
 			for (ptr = tmp->groups; ptr != NULL ; ptr = ptr->next) {
 				group = ptr->data;
 				if (group->ns == ns)
@@ -536,25 +790,14 @@ gsf_xml_in_doc_extend (GsfXMLInDoc  *doc,
 				group->ns = ns;
 				tmp->groups = g_slist_prepend (tmp->groups, group);
 			}
-			group->elem = g_slist_prepend (group->elem, real_node);
-		} else if (strcmp (node->id, node->parent_id)) {
-			g_warning ("Parent ID '%s' unknown", node->parent_id);
-			return;
+			group->elem = g_slist_prepend (group->elem, node);
+		} else if (strcmp (node->pub.id, node->pub.parent_id)) {
+			g_warning ("Parent ID '%s' unknown", node->pub.parent_id);
+			continue;
 		}
-
-		/* WARNING VILE HACK :
-		 * The api in 1.8.2 passed has_content as a boolean.  It's too
-		 * late to change it but we need more contol.  We edit the bool
-		 * here to be GSF_CONTENT_NONE, GSF_CONTENT_ROOT and add a
-		 * mechanism to edit the flag later */
-		if (node->has_content != 0 &&
-		    node->has_content != GSF_XML_SHARED_CONTENT)
-			node->has_content = GSF_XML_CONTENT;
-
-		node->parent_initialized = TRUE;
 	}
 
-	g_hash_table_destroy (symbols);
+	return doc;
 }
 
 /**
@@ -573,61 +816,66 @@ gsf_xml_in_doc_set_unknown_handler (GsfXMLInDoc *doc,
 }
 
 /**
- * gsf_xml_in_doc_free :
- * @doc :
+ * gsf_xml_in_push_state :
+ * @xin : #GsfXMLIn
+ * @doc : #GsfXMLInDoc
+ * @new_state :
+ * @dtor : #GsfXMLInExtDtor
+ * @attrs :
  *
- * Free up resources allocated by gsf_xml_in_doc_prep
+ * Take the first node from @doc as the current node and call it's start handler.
  **/
 void
-gsf_xml_in_doc_free (GsfXMLInDoc *doc)
+gsf_xml_in_push_state (GsfXMLIn *xin, GsfXMLInDoc const *doc,
+		       gpointer new_state, GsfXMLInExtDtor dtor,
+		       xmlChar const **attrs)
 {
-	GSList *ptr;
-	GsfXMLInNode *node;
-	GsfXMLInNodeGroup *group;
-
+	GsfXMLInInternal *state = (GsfXMLInInternal *)xin;
+	GsfXMLInExtension *ext;
+	
+	g_return_if_fail (xin != NULL);
 	g_return_if_fail (doc != NULL);
-	g_return_if_fail (doc->root != NULL);
-	g_return_if_fail (doc->ns_by_id != NULL);
+	g_return_if_fail (doc->root_node != NULL);
 
-	for (node = doc->root; node->id != NULL ; node++ ) {
-		for (ptr = node->groups; ptr != NULL ; ptr = ptr->next) {
-			group = ptr->data;
-			g_slist_free (group->elem);
-			g_free (group);
-		}
-		g_slist_free (node->groups);
-		node->groups = NULL;
-	}
+	ext = g_new (GsfXMLInExtension, 1);
+	ext->old_doc	= xin->doc;
+	ext->old_state	= xin->user_state;
+	ext->dtor	= dtor;
+	ext->from_unknown = state->from_unknown_handler;
 
-	g_ptr_array_free (doc->ns_by_id, TRUE);
-	g_free (doc);
+	xin->doc = doc;
+	xin->user_state = new_state;
+
+	push_child (state, &doc->root_node->pub, NULL, attrs, ext);
 }
 
 /**
- * gsf_xml_in_parse :
- * @state :
+ * gsf_xml_in_doc_parse :
+ * @doc :
  * @input :
+ * @user_state :
  *
  * Read an xml document from @input and parse based on the the descriptor in
- * @state::doc
+ * @doc
  *
  * returns FALSE on error
  **/
 gboolean
-gsf_xml_in_parse (GsfXMLIn *state, GsfInput *input)
+gsf_xml_in_doc_parse (GsfXMLInDoc *doc, GsfInput *input, gpointer user_state)
 {
-	xmlParserCtxt *ctxt;
+	xmlParserCtxt	*ctxt;
+	GsfXMLInInternal state;
 	gboolean res;
 
-	g_return_val_if_fail (state != NULL, FALSE);
-	g_return_val_if_fail (state->doc != NULL, FALSE);
-	g_return_val_if_fail (GSF_IS_INPUT (input), FALSE);
+	g_return_val_if_fail (doc != NULL, FALSE);
 
-	ctxt = gsf_xml_parser_context_full (input, &gsfXMLInParser, state);
+	ctxt = gsf_xml_parser_context_full (input, &gsfXMLInParser, &state);
+	if (ctxt == NULL)
+		return FALSE;
 
-	g_return_val_if_fail (ctxt != NULL, FALSE);
-
-	state->content = g_string_sized_new (128);
+	state.pub.doc = doc;
+	state.pub.user_state = user_state;
+	state.pub.content = g_string_sized_new (128);
 	xmlParseDocument (ctxt);
 	res = ctxt->wellFormed;
 	xmlFreeParserCtxt (ctxt);
@@ -637,7 +885,7 @@ gsf_xml_in_parse (GsfXMLIn *state, GsfInput *input)
 
 /**
  * gsf_xml_in_check_ns :
- * @state :
+ * @xin :
  * @str :
  * @ns_id :
  * 
@@ -647,8 +895,9 @@ gsf_xml_in_parse (GsfXMLIn *state, GsfInput *input)
  * otherwise NULL.
  **/
 char const *
-gsf_xml_in_check_ns (GsfXMLIn const *state, char const *str, unsigned int ns_id)
+gsf_xml_in_check_ns (GsfXMLIn const *xin, char const *str, unsigned int ns_id)
 {
+	GsfXMLInInternal const *state = (GsfXMLInInternal const *)xin;
 	GsfXMLInNSInstance *inst;
 	if (state->ns_by_id->len <= ns_id)
 		return NULL;
@@ -661,7 +910,7 @@ gsf_xml_in_check_ns (GsfXMLIn const *state, char const *str, unsigned int ns_id)
 
 /**
  * gsf_xml_in_namecmp :
- * @state : The #GsfXMLIn we are reading from.
+ * @xin   : The #GsfXMLIn we are reading from.
  * @str   : The potentially namespace qualified node name.
  * @ns_id : The name space id to check
  * @name  : The target node name
@@ -669,10 +918,12 @@ gsf_xml_in_check_ns (GsfXMLIn const *state, char const *str, unsigned int ns_id)
  * Returns TRUE if @str == @ns_id:@name according to @state.
  **/
 gboolean
-gsf_xml_in_namecmp (GsfXMLIn const *state, char const *str,
+gsf_xml_in_namecmp (GsfXMLIn const *xin, char const *str,
 		    unsigned int ns_id, char const *name)
 {
+	GsfXMLInInternal const *state = (GsfXMLInInternal const *)xin;
 	GsfXMLInNSInstance *inst;
+
 	if (state->ns_by_id->len <= ns_id)
 		return FALSE;
 	if (NULL == (inst = g_ptr_array_index (state->ns_by_id, ns_id)))
@@ -696,7 +947,7 @@ struct _GsfXMLOut {
 	GsfOutput	 *output;
 	char		 *doc_type;
 	GSList		 *stack;
-	GsfXMLOutState state;
+	GsfXMLOutState	  state;
 	unsigned   	  indent;
 	gboolean	  needs_header;
 };
