@@ -2,7 +2,7 @@
 /*
  * gsf-libxml.h: Utility wrappers for using gsf with libxml
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2005 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -41,51 +41,37 @@ int	       gsf_xmlDocFormatDump   (GsfOutput  *output,
 				       gboolean    format);
 
 /****************************************************************************/
-/* Simplified GSF based xml import (based on libxml2 SAX) */
-typedef struct _GsfXMLBlob	GsfXMLBlob;
-
-typedef struct _GsfXMLIn	GsfXMLIn;
-typedef struct _GsfXMLInDoc	GsfXMLInDoc;
-typedef struct _GsfXMLInNode	GsfXMLInNode;
-typedef struct _GsfXMLInNS	GsfXMLInNS;
-
+/* Simplified wrapper to SAX based xml import */
+typedef struct _GsfXMLIn		GsfXMLIn;
+typedef struct _GsfXMLInDoc		GsfXMLInDoc;
+typedef struct _GsfXMLInNode		GsfXMLInNode;
+typedef struct _GsfXMLInNS		GsfXMLInNS;
+typedef struct _GsfXMLBlob		GsfXMLBlob;
 typedef enum {
-	GSF_XML_NO_CONTENT,
+	GSF_XML_NO_CONTENT = FALSE,
 	GSF_XML_CONTENT,
 	GSF_XML_SHARED_CONTENT
 } GsfXMLContent;
+typedef gboolean (*GsfXMLInUnknownFunc) (GsfXMLIn *xin,
+					 xmlChar const *elem, xmlChar const **attrs);
+typedef void     (*GsfXMLInExtDtor)     (GsfXMLIn *xin, gpointer old_state);
 
-typedef gboolean (*GsfXMLInUnknownFunc) (GsfXMLIn *state, xmlChar const *elem, xmlChar const **attrs);
 struct _GsfXMLIn {
-	GsfXMLInDoc  const *doc;	/* init before parsing */
-
-    /* look but do not change */
-	GsfXMLInNode const *node;	/* current node */
-	GSList	 	   *state_stack;
-
-	GsfXMLInNS   const *default_ns;	/* optionally NULL */
-	GSList	 	   *ns_stack;
-
-	GString		*content;
-	gint		 unknown_depth;	/* handle recursive unknown tags */
-	GHashTable	*ns_prefixes;	/* current ns prefixes */
-	GPtrArray	*ns_by_id;		/* indexed by id */
+	/* public state : read only */
+	gpointer	    user_state;
+	GString		   *content;
+	GsfXMLInDoc  const *doc;
+	GsfXMLInNode const *node;	/* current node (not on the stack) */
+	GSList	 	   *node_stack;	/* stack of GsfXMLInNode */
 };
 
 struct _GsfXMLInNode {
-	char const *id;
+	char const *id;		/* unique in the entire tree */
 	int	    ns_id;
 	char const *name;
 	char const *parent_id;
-	gboolean parent_initialized;
-	GSList *groups;
-
-	unsigned	has_content;
-	gboolean	deprecated_unused_allow_unknown; /* remains here for binary compat */
-	gboolean	check_children_for_ns;
-
-	void (*start) (GsfXMLIn *state, xmlChar const **attrs);
-	void (*end)   (GsfXMLIn *state, GsfXMLBlob *unknown);
+	void (*start) (GsfXMLIn *xin, xmlChar const **attrs);
+	void (*end)   (GsfXMLIn *xin, GsfXMLBlob *unknown);
 
 	union {
 		int	    v_int;
@@ -93,6 +79,10 @@ struct _GsfXMLInNode {
 		gpointer    v_blob;
 		char const *v_str;
 	} user_data;
+	GsfXMLContent has_content;
+
+	unsigned int check_children_for_ns : 1;
+	unsigned int share_children_with_parent : 1;
 };
 
 struct _GsfXMLInNS {
@@ -101,31 +91,36 @@ struct _GsfXMLInNS {
 };
 
 #define GSF_XML_IN_NS(id, uri) \
-{ uri, id}
+{ uri, id }
 
 #define GSF_XML_IN_NODE_FULL(parent_id, id, ns, name, has_content, 	\
-			     deprecated_unused_allow_unknown, check_ns, start, end, user)	\
+			     share_children_with_parent, check_ns, start, end, user)	\
 {									\
-	#id, ns, name, #parent_id, FALSE, NULL,				\
-	has_content, deprecated_unused_allow_unknown, check_ns, start, end, { user } 	\
+	#id, ns, name, #parent_id, start, end, { user }, has_content,	\
+	check_ns, share_children_with_parent,				\
 }
 
 #define GSF_XML_IN_NODE(parent_id, id, ns, name, has_content, start, end) \
 	GSF_XML_IN_NODE_FULL(parent_id, id, ns, name, has_content,	  \
 			     FALSE, FALSE, start, end, 0)
+#define GSF_XML_IN_NODE_END	\
+	{ NULL, 0, NULL, NULL, NULL, NULL, { 0 }, GSF_XML_NO_CONTENT, FALSE, FALSE }
 
-GsfXMLInDoc *gsf_xml_in_doc_new	 (GsfXMLInNode *root, GsfXMLInNS *ns);
-void	     gsf_xml_in_doc_free (GsfXMLInDoc *doc);
-void	     gsf_xml_in_doc_extend (GsfXMLInDoc  *doc,
-				    GsfXMLInNode *nodes);
+GsfXMLInDoc *gsf_xml_in_doc_new	   (GsfXMLInNode const *nodes, GsfXMLInNS const *ns);
+void	     gsf_xml_in_doc_free   (GsfXMLInDoc *doc);
+gboolean     gsf_xml_in_doc_parse  (GsfXMLInDoc *doc, GsfInput *input,
+				    gpointer user_state);
 void	     gsf_xml_in_doc_set_unknown_handler (GsfXMLInDoc *doc,
-						 GsfXMLInUnknownFunc handler);
+				    GsfXMLInUnknownFunc handler);
 
-gboolean    gsf_xml_in_parse	 (GsfXMLIn *state, GsfInput *input);
-char const *gsf_xml_in_check_ns	 (GsfXMLIn const *state, char const *str,
-				  unsigned int ns_id);
-gboolean    gsf_xml_in_namecmp	 (GsfXMLIn const *state, char const *str,
-				  unsigned int ns_id, char const *name);
+void	     gsf_xml_in_push_state (GsfXMLIn *xin, GsfXMLInDoc const *doc,
+				    gpointer new_state, GsfXMLInExtDtor dtor,
+				    xmlChar const **attrs);
+
+char const  *gsf_xml_in_check_ns   (GsfXMLIn const *xin, char const *str,
+				    unsigned int ns_id);
+gboolean     gsf_xml_in_namecmp	   (GsfXMLIn const *xin, char const *str,
+				    unsigned int ns_id, char const *name);
 
 /****************************************************************************/
 /* Simplified GSF based xml export (does not use libxml) */
@@ -169,6 +164,10 @@ void gsf_xml_out_add_base64		(GsfXMLOut *xml, char const *id,
 void gsf_xml_out_add_enum               (GsfXMLOut *xml, char const *id,
 					 GType etype, gint val);
 
+/****************************************************************************/
+/* Some general utilities */
+gboolean gsf_xml_gvalue_from_str (GValue *res, GType t, char const *str);
+
 G_END_DECLS
 
 #endif /* GSF_LIBXML_H */
