@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -23,6 +23,7 @@
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input.h>
 #include <gobject/gvaluecollector.h>
+#include <glib/gi18n-lib.h>
 
 #include <ctype.h>
 #include <stdio.h>
@@ -50,6 +51,11 @@ static void base64_init (void);
 void
 gsf_init (void)
 {
+#ifdef ENABLE_NLS
+	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+#endif
+
 	g_type_init ();
 	base64_init ();
 }
@@ -67,20 +73,22 @@ gsf_shutdown (void)
 void
 gsf_init_dynamic (GTypeModule *module)
 {
+	(void)module;
 }
 
 void
 gsf_shutdown_dynamic (GTypeModule *module)
 {
+	(void)module;
 }
 
 static void
-gsf_mem_dump_full (guint8 const *ptr, size_t len, guint offset)
+gsf_mem_dump_full (guint8 const *ptr, size_t len, gsf_off_t offset)
 {
-	size_t i, j, off;
+	gsf_off_t i, j, off;
 
 	for (i = 0 ; i < (len+15)/16 ; i++) {
-		g_print ("%8x | ", i*16 + offset);
+		g_print ("%8lx | ", (long)(i*16 + offset));
 		for (j = 0;j < 16; j++) {
 			off = j + (i << 4);
 			off<len ? g_print("%2x ", ptr[off]) : g_print("XX ");
@@ -117,7 +125,8 @@ gsf_mem_dump (guint8 const *ptr, size_t len)
 void
 gsf_input_dump (GsfInput *input, gboolean dump_as_hex)
 {
-	size_t size, count, offset = 0;
+	gsf_off_t offset = 0;
+	size_t size, count;
 	guint8 const *data;
 
 	/* read in small blocks to excercise things */
@@ -399,8 +408,8 @@ gsf_filename_to_utf8 (char const *filename, gboolean quoted)
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 /* dont touch this file without my permission - Michael */
@@ -735,3 +744,26 @@ gsf_property_settings_free (GParameter *params,
 		g_value_unset (&params[n_params].value);
 	g_free (params);
 }
+
+
+
+/* Errors */
+
+/**
+ * gsf_error_quark:
+ *
+ * Returns the #GQuark used to identify libgsf errors in #GError structures.
+ * Specific error codes come from the #GsfError enumeration.
+ * 
+ * Return value: A #GQuark.
+ **/
+GQuark
+gsf_error_quark (void)
+{
+	static GQuark quark;
+
+	if (quark == 0)
+		quark = g_quark_from_static_string ("gsf-error-quark");
+
+	return quark;
+}
