@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -54,19 +54,19 @@ double  gsf_le_get_double  (void const *p);
 	((*((guint8 *)(p) + 0) = ((dat)        & 0xff)),\
 	 (*((guint8 *)(p) + 1) = ((dat) >>  8) & 0xff))
 #define GSF_LE_SET_GUINT32(p, dat)				\
-	((*((guint8 *)(p) + 0) = ((dat))       & 0xff),	\
-	 (*((guint8 *)(p) + 1) = ((dat) >>  8) & 0xff),	\
-	 (*((guint8 *)(p) + 2) = ((dat) >> 16) & 0xff),	\
-	 (*((guint8 *)(p) + 3) = ((dat) >> 24) & 0xff))
+	((*((guint8 *)(p) + 0) = (guchar) ((dat))       & 0xff),	\
+	 (*((guint8 *)(p) + 1) = (guchar) ((dat) >>  8) & 0xff),	\
+	 (*((guint8 *)(p) + 2) = (guchar) ((dat) >> 16) & 0xff),	\
+	 (*((guint8 *)(p) + 3) = (guchar) ((dat) >> 24) & 0xff))
 #define GSF_LE_SET_GUINT64(p, dat)			\
-	((*((guint8 *)(p) + 0) = ((dat))       & 0xff),	\
-	 (*((guint8 *)(p) + 1) = ((dat) >>  8) & 0xff),	\
-	 (*((guint8 *)(p) + 2) = ((dat) >> 16) & 0xff),	\
-	 (*((guint8 *)(p) + 3) = ((dat) >> 24) & 0xff),	\
-	 (*((guint8 *)(p) + 4) = ((dat) >> 32) & 0xff),	\
-	 (*((guint8 *)(p) + 5) = ((dat) >> 40) & 0xff),	\
-	 (*((guint8 *)(p) + 6) = ((dat) >> 48) & 0xff),	\
-	 (*((guint8 *)(p) + 7) = ((dat) >> 56) & 0xff))
+	((*((guint8 *)(p) + 0) = (guchar) ((dat))       & 0xff),	\
+	 (*((guint8 *)(p) + 1) = (guchar) ((dat) >>  8) & 0xff),	\
+	 (*((guint8 *)(p) + 2) = (guchar) ((dat) >> 16) & 0xff),	\
+	 (*((guint8 *)(p) + 3) = (guchar) ((dat) >> 24) & 0xff),	\
+	 (*((guint8 *)(p) + 4) = (guchar) ((dat) >> 32) & 0xff),	\
+	 (*((guint8 *)(p) + 5) = (guchar) ((dat) >> 40) & 0xff),	\
+	 (*((guint8 *)(p) + 6) = (guchar) ((dat) >> 48) & 0xff),	\
+	 (*((guint8 *)(p) + 7) = (guchar) ((dat) >> 56) & 0xff))
 #define GSF_LE_SET_GINT8(p,dat) GSF_LE_SET_GUINT8((p),(dat))
 #define GSF_LE_SET_GINT16(p,dat) GSF_LE_SET_GUINT16((p),(dat))
 #define GSF_LE_SET_GINT32(p,dat) GSF_LE_SET_GUINT32((p),(dat))
@@ -123,6 +123,19 @@ void        gsf_property_settings_collect (GType object_type,
 void        gsf_property_settings_free (GParameter *params,
 					size_t n_params);
 
+
+
+/* Errors */
+
+#define GSF_ERROR (gsf_error_quark ())
+
+typedef enum {
+	GSF_ERROR_OUT_OF_MEMORY,
+	GSF_ERROR_INVALID_DATA
+} GsfError;
+
+GQuark gsf_error_quark (void);
+
 
 G_END_DECLS
 
